"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.call = void 0;
const util_1 = require("util");
const child_process_1 = require("child_process");
const path_1 = __importDefault(require("path"));
const internal_ip_1 = __importDefault(require("internal-ip"));
const execP = (0, util_1.promisify)(child_process_1.exec);
const _hasura = path_1.default.normalize(`${__dirname}/../../hasura`);
const _deeplinks = path_1.default.normalize(`${__dirname}/../`);
const handleEnvWindows = (k, envs, envString) => envString + ` set ${k} ${envs[k]} &&`;
const handleEnvUnix = (k, envs, envString) => envString + ` export ${k}=${envs[k]} &&`;
const handleEnv = process.platform === "win32" ? handleEnvWindows : handleEnvUnix;
const generateEnvs = (envs) => {
    let envsString = '';
    Object.keys(envs).forEach(k => envsString += handleEnv(k, envs, envsString));
    return envsString;
};
function call(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const envs = Object.assign(Object.assign({}, options.envs), { DOCKERHOST: yield internal_ip_1.default.v4() });
        let envsString = generateEnvs(envs);
        try {
            if (options.operation === 'run') {
                let str = `${envsString} cd ${path_1.default.normalize(`${_hasura}/local/`)} && npm run docker && npx -q wait-on tcp:8080 && cd ${_deeplinks} && npm run migrate`;
                const { stdout, stderr } = yield execP(str);
                return Object.assign(Object.assign({}, options), { envs, str, stdout, stderr });
            }
            if (options.operation === 'sleep') {
                let str = `${envsString} cd ${path_1.default.normalize(`${_hasura}/local/`)} && docker-compose down`;
                const { stdout, stderr } = yield execP(str);
                return Object.assign(Object.assign({}, options), { envs, str, stdout, stderr });
            }
            if (options.operation === 'reset') {
                let str = `${envsString} cd ${path_1.default.normalize(`${_hasura}/local/`)} && docker-compose down && docker container prune -f && docker system prune --volumes -f && cd ${_deeplinks} && npx rimraf .migrate`;
                const { stdout, stderr } = yield execP(str);
                return Object.assign(Object.assign({}, options), { envs, str, stdout, stderr });
            }
        }
        catch (error) {
            return Object.assign(Object.assign({}, options), { envs, error });
        }
        return options;
    });
}
exports.call = call;
;
//# sourceMappingURL=engine.js.map