import { ApolloClient } from '@apollo/client';
import { LinksResult, Link } from './minilinks';
export interface PackagerPackageIdentifier {
    name: string;
    version?: string;
    uri?: string;
    type?: string;
    options?: any;
}
export interface PackagerPackage {
    package: PackagerPackageIdentifier;
    data: PackagerPackageItem[];
    dependencies?: {
        [id: number]: PackagerPackageIdentifier;
    };
    strict?: boolean;
    errors?: PackagerError[];
}
export interface PackagerPackageItem {
    id: number | string;
    type?: number | string;
    from?: number | string;
    to?: number | string;
    value?: PackagerValue;
    package?: {
        dependencyId: number;
        containValue: string;
    };
}
export interface PackagerValue {
    [key: string]: any;
}
export declare type PackagerError = any;
export interface PackagerImportResult {
    errors?: PackagerError[];
    ids?: number[];
}
export declare type PackagerMutated = {
    [index: number]: boolean;
};
export interface PackagerExportOptions {
    packageLinkId: number;
}
export interface PackagerLink extends Link<any> {
    value?: any;
}
export declare function sort(pckg: PackagerPackage, data: PackagerPackageItem[], errors?: PackagerError[]): {
    sorted: any[];
};
export declare class Packager {
    pckg: PackagerPackage;
    client: ApolloClient<any>;
    constructor(client: ApolloClient<any>);
    fetchPackageNamespaceId(name: string): Promise<number>;
    fetchDependenciedLinkId(pckg: PackagerPackage, dependedLink: PackagerPackageItem): Promise<number>;
    fetchTableNamesValuedToTypeId(types: number[], errors: PackagerError[]): Promise<{
        [type: string]: string;
    }>;
    insertItem(id: number, items: PackagerPackageItem[], item: PackagerPackageItem, errors: PackagerError[], mutated: PackagerMutated): Promise<void>;
    insertItems(pckg: PackagerPackage, data: PackagerPackageItem[], counter: number, dependedLinks: PackagerPackageItem[], errors?: PackagerError[], mutated?: {
        [index: number]: boolean;
    }): Promise<{
        ids: number[];
    }>;
    importNamespace(): Promise<void>;
    import(pckg: PackagerPackage): Promise<PackagerImportResult>;
    selectLinks(options: PackagerExportOptions): Promise<LinksResult<PackagerLink>>;
    deserialize(pckg: PackagerPackage, errors?: PackagerError[]): Promise<{
        data: PackagerPackageItem[];
        errors?: PackagerError[];
        counter: number;
        dependedLinks: PackagerPackageItem[];
    }>;
    serialize(globalLinks: LinksResult<PackagerLink>, options: PackagerExportOptions, pckg: PackagerPackage): Promise<void>;
    export(options: PackagerExportOptions): Promise<PackagerPackage>;
}
