"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Packager = exports.sort = void 0;
const debug_1 = __importDefault(require("debug"));
const global_ids_1 = require("./global-ids");
const gql_1 = require("./gql");
const query_1 = require("./gql/query");
const minilinks_1 = require("./minilinks");
const promise_1 = require("./promise");
const reserve_1 = require("./reserve");
const debug = (0, debug_1.default)('deeplinks:packager');
function sort(pckg, data, errors = []) {
    let sorted = [];
    if (pckg.strict) {
        sorted = data;
    }
    else {
        for (let i = 0; i < data.length; i++) {
            const item = data[i];
            if (item === null || item === void 0 ? void 0 : item.type) {
                const first = sorted.findIndex((l, index) => l.from == item.id || l.to == item.id || l.type == item.id);
                const max = sorted.reduce((p, l, index) => l.id == item.type || l.id == item.from || l.id === item.to ? index : p, 0);
                sorted.splice((!!~first && first < max ? first : max) + 1, 0, item);
            }
            else {
                const max = sorted.reduce((p, l, index) => l.id == item.id ? index : p, 0);
                sorted.splice((max) + 1, 0, item);
            }
        }
    }
    return { sorted };
}
exports.sort = sort;
class Packager {
    constructor(client) {
        this.client = client;
        global.packager = this;
    }
    fetchPackageNamespaceId(name) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const tables = yield this.fetchTableNamesValuedToTypeId([global_ids_1.GLOBAL_ID_PACKAGE_NAMESPACE], []);
                const q = yield this.client.query((0, query_1.generateQuery)({
                    queries: [
                        (0, query_1.generateQueryData)({ tableName: tables[global_ids_1.GLOBAL_ID_PACKAGE_NAMESPACE], returning: `id: link_id`, variables: { where: {
                                    value: { _eq: name },
                                } } }),
                    ],
                    name: 'LOAD_PACKAGE_NAMESPACE',
                }));
                return (_d = (_c = (_b = (_a = (q)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.q0) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.id;
            }
            catch (error) {
                debug('fetchPackageNamespaceId error', error);
            }
            return 0;
        });
    }
    fetchDependenciedLinkId(pckg, dependedLink) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const tables = yield this.fetchTableNamesValuedToTypeId([global_ids_1.GLOBAL_ID_CONTAIN, global_ids_1.GLOBAL_ID_PACKAGE], []);
                const packageName = (_c = (_a = pckg === null || pckg === void 0 ? void 0 : pckg.dependencies) === null || _a === void 0 ? void 0 : _a[(_b = dependedLink === null || dependedLink === void 0 ? void 0 : dependedLink.package) === null || _b === void 0 ? void 0 : _b.dependencyId]) === null || _c === void 0 ? void 0 : _c.name;
                const packageTableName = `table${tables[global_ids_1.GLOBAL_ID_PACKAGE]}`;
                const q = yield this.client.query((0, query_1.generateQuery)({
                    queries: [
                        (0, query_1.generateQueryData)({ tableName: `table${tables[global_ids_1.GLOBAL_ID_CONTAIN]}`, returning: `id link { id: to_id }`, variables: { where: {
                                    value: { _eq: (_d = dependedLink === null || dependedLink === void 0 ? void 0 : dependedLink.package) === null || _d === void 0 ? void 0 : _d.containValue },
                                    link: { from: { [packageTableName]: { value: { _eq: packageName } } } },
                                } } }),
                    ],
                    name: 'FETCH_DEPENDENCIES_LINKS_IDS',
                }));
                return ((_j = (_h = (_g = (_f = (_e = (q)) === null || _e === void 0 ? void 0 : _e.data) === null || _f === void 0 ? void 0 : _f.q0) === null || _g === void 0 ? void 0 : _g[0]) === null || _h === void 0 ? void 0 : _h.link) === null || _j === void 0 ? void 0 : _j.id) || 0;
            }
            catch (error) {
                debug('fetchDependenciedLinkId error', error);
            }
            return 0;
        });
    }
    fetchTableNamesValuedToTypeId(types, errors) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const result = {};
            try {
                const q = yield this.client.query((0, query_1.generateQuery)({
                    queries: [
                        (0, query_1.generateQueryData)({ tableName: 'links', returning: `id values: out(where: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_TABLE_VALUE} } }) { id: to_id }`, variables: { where: {
                                    type_id: { _eq: global_ids_1.GLOBAL_ID_TABLE },
                                    out: { type_id: { _eq: global_ids_1.GLOBAL_ID_TABLE_VALUE }, to_id: { _in: types } },
                                } } }),
                    ],
                    name: 'LOAD_TYPES_TABLES_HASH',
                }));
                const tables = (_b = (_a = (q)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.q0;
                for (let t = 0; t < tables.length; t++) {
                    const table = tables[t];
                    for (let i = 0; i < table.values.length; i++) {
                        const type = table.values[i].id;
                        if (types.includes(type))
                            result[type] = table.id;
                    }
                }
            }
            catch (error) {
                debug('fetchTableNamesValuedToTypeId error', error);
                errors.push(error);
            }
            return result;
        });
    }
    insertItem(id, items, item, errors, mutated) {
        return __awaiter(this, void 0, void 0, function* () {
            debug('insertItem', { id }, item);
            try {
                if (item.type) {
                    const linkInsert = yield this.client.mutate((0, gql_1.generateSerial)({
                        actions: [(0, gql_1.insertMutation)('links', { objects: { id, type_id: item.type, from_id: item.from || 0, to_id: item.to || 0 } })],
                        name: 'IMPORT_PACKAGE_LINK',
                    }));
                    if (linkInsert === null || linkInsert === void 0 ? void 0 : linkInsert.errors) {
                        errors.push(linkInsert === null || linkInsert === void 0 ? void 0 : linkInsert.errors);
                    }
                }
                if (item.type || item.package) {
                    mutated[item.id] = true;
                    item.id = id;
                    items.filter(i => (i.id === item.id && (!i._))).forEach(l => l.id = id);
                    items.filter(i => (i.from === item.id && (!i._))).forEach(l => l.from = id);
                    items.filter(i => (i.to === item.id && (!i._))).forEach(l => l.to = id);
                    items.filter(i => (i.type === item.id && (!i._))).forEach(l => l.type = id);
                }
                debug('insertItem promise', id);
                yield (0, promise_1.awaitPromise)({ id, client: this.client });
                debug('insertItem promise awaited', id);
                if (item.value && !item.package) {
                    debug('insertItem value', id, item);
                    let valueLink;
                    if (!item.type) {
                        valueLink = items.find(i => i.id === item.id && !!i.type);
                        debug('insertItem .value', item, valueLink);
                    }
                    else {
                        valueLink = item;
                    }
                    if (!valueLink) {
                        errors.push(`Link ${JSON.stringify(item)} for value not founded.`);
                    }
                    else {
                        const tables = yield this.fetchTableNamesValuedToTypeId([+valueLink.type], errors);
                        debug('insertItem tables', tables);
                        if (!tables[valueLink.type])
                            errors.push(`Table for type ${valueLink.type} for link ${valueLink.id} not founded for insert ${JSON.stringify(item)}.`);
                        const valueInsert = yield this.client.mutate((0, gql_1.generateSerial)({
                            actions: [(0, gql_1.insertMutation)(`table${tables[valueLink.type]}`, { objects: Object.assign({ link_id: valueLink.id }, item.value) })],
                            name: 'IMPORT_PACKAGE_VALUE',
                        }));
                        debug('insertItem valueInsert', valueInsert);
                        if (valueInsert === null || valueInsert === void 0 ? void 0 : valueInsert.errors)
                            errors.push(valueInsert === null || valueInsert === void 0 ? void 0 : valueInsert.errors);
                    }
                }
            }
            catch (error) {
                debug('insertItem error', error);
                errors.push(error);
            }
            return;
        });
    }
    insertItems(pckg, data, counter, dependedLinks, errors = [], mutated = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const ids = yield (0, reserve_1.reserve)({
                    client: this.client,
                    count: (counter - dependedLinks.length) + 2,
                });
                for (let i = 0; i < dependedLinks.length; i++) {
                    const item = dependedLinks[i];
                    const id = yield this.fetchDependenciedLinkId(pckg, item);
                    if (!id)
                        throw new Error(`Cant find id for ${item.id}`);
                    yield this.insertItem(id, data, item, errors, mutated);
                    if (errors.length)
                        return;
                }
                for (let i = 0; i < data.length; i++) {
                    const item = data[i];
                    if (!item.package) {
                        const id = ids[+(item.id) - 1];
                        if (!id)
                            throw new Error(`Cant find id for ${item.id} ${data.filter(i => i.id === item.id)}`);
                        yield this.insertItem(id, data, item, errors, mutated);
                        if (errors.length)
                            return;
                    }
                }
                return { ids };
            }
            catch (error) {
                debug('insertItems error', error);
                errors.push(error);
            }
            return { ids: [] };
        });
    }
    importNamespace() {
        return __awaiter(this, void 0, void 0, function* () {
        });
    }
    import(pckg) {
        return __awaiter(this, void 0, void 0, function* () {
            const errors = [];
            try {
                const { data, counter, dependedLinks } = yield this.deserialize(pckg, errors);
                if (errors.length)
                    return { errors };
                const { sorted } = sort(pckg, data, errors);
                if (errors.length)
                    return { errors };
                const mutated = {};
                const { ids } = yield this.insertItems(pckg, sorted, counter, dependedLinks, errors, mutated);
                if (errors.length)
                    return { errors };
                yield this.importNamespace();
                return { ids, errors };
            }
            catch (error) {
                debug('insertItems error', error);
                errors.push(error);
            }
            return { ids: [], errors };
        });
    }
    selectLinks(options) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.query((0, query_1.generateQuery)({
                queries: [
                    (0, query_1.generateQueryData)({ tableName: 'links', returning: `
          id type_id from_id to_id value
          type {
            id value
            contains: in(where: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_CONTAIN} }, from: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_PACKAGE} } } }) {
              id value
              package: from {
                id value
              }
            }
          }
          from {
            id
            contains: in(where: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_CONTAIN} }, from: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_PACKAGE} } } }) {
              id value
              package: from {
                id value
              }
            }
          }
          to {
            id
            contains: in(where: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_CONTAIN} }, from: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_PACKAGE} } } }) {
              id value
              package: from {
                id value
              }
            }
          }
        `, variables: { where: {
                                _or: [
                                    { id: { _eq: options.packageLinkId } },
                                    { type_id: { _eq: global_ids_1.GLOBAL_ID_CONTAIN }, from: { id: { _eq: options.packageLinkId } } },
                                    { in: { type_id: { _eq: global_ids_1.GLOBAL_ID_CONTAIN }, from: { id: { _eq: options.packageLinkId } } } },
                                ]
                            } } }),
                ],
                name: 'LOAD_PACKAGE_LINKS',
            }));
            return (0, minilinks_1.minilinks)((_b = (_a = (result)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.q0);
        });
    }
    deserialize(pckg, errors = []) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = pckg.data.map(l => (Object.assign(Object.assign({}, l), { value: l.value ? Object.assign({}, l.value) : undefined })));
            const containsHash = {};
            let counter = 0;
            const dependedLinks = [];
            for (let l = 0; l < data.length; l++) {
                const item = data[l];
                const local = item.id;
                if (containsHash[local])
                    item.id = containsHash[local];
                else
                    containsHash[local] = item.id = ++counter;
                if (item.package)
                    dependedLinks.push(item);
            }
            for (let l = 0; l < data.length; l++) {
                const item = data[l];
                if (item.type)
                    item.type = containsHash[item.type];
                if (item.from)
                    item.from = containsHash[item.from] || 0;
                if (item.to)
                    item.to = containsHash[item.to] || 0;
            }
            const packageId = ++counter;
            data.push({
                id: packageId,
                type: global_ids_1.GLOBAL_ID_PACKAGE,
                value: { value: pckg.package.name },
                _: true,
            });
            let namespaceId = yield this.fetchPackageNamespaceId(pckg.package.name);
            if (!namespaceId) {
                namespaceId = ++counter;
                data.push({
                    id: namespaceId,
                    type: global_ids_1.GLOBAL_ID_PACKAGE_NAMESPACE,
                    value: { value: pckg.package.name },
                    _: true,
                });
                data.push({
                    id: ++counter,
                    type: global_ids_1.GLOBAL_ID_PACKAGE_ACTIVE,
                    to: packageId,
                    from: namespaceId,
                    _: true,
                });
            }
            data.push({
                id: ++counter,
                type: global_ids_1.GLOBAL_ID_CONTAIN,
                from: namespaceId,
                to: packageId,
                _: true,
            });
            const containsArray = Object.keys(containsHash);
            for (let c = 0; c < containsArray.length; c++) {
                const contain = containsArray[c];
                data.push({
                    id: ++counter,
                    type: global_ids_1.GLOBAL_ID_CONTAIN,
                    from: packageId,
                    to: containsHash[contain],
                    value: { value: contain },
                    _: true,
                });
            }
            return { data, errors, counter, dependedLinks };
        });
    }
    serialize(globalLinks, options, pckg) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11;
        return __awaiter(this, void 0, void 0, function* () {
            let counter = 1;
            let dependenciesCounter = 1;
            const dependencies = {};
            const dependencedPackages = {};
            const handledDepLinks = {};
            const containsByTo = {};
            const links = [];
            for (let l = 0; l < globalLinks.links.length; l++) {
                const link = globalLinks.links[l];
                if (!!link.type_id && !globalLinks.byId[link.type_id] && !((link === null || link === void 0 ? void 0 : link.type_id) === global_ids_1.GLOBAL_ID_CONTAIN && ((_a = link === null || link === void 0 ? void 0 : link.from) === null || _a === void 0 ? void 0 : _a.id) === options.packageLinkId) && !((link === null || link === void 0 ? void 0 : link.id) === options.packageLinkId)) {
                    const name = (_f = (_e = (_d = (_c = (_b = link === null || link === void 0 ? void 0 : link.type) === null || _b === void 0 ? void 0 : _b.contains) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.package) === null || _e === void 0 ? void 0 : _e.value) === null || _f === void 0 ? void 0 : _f.value;
                    const foundedDep = dependencedPackages[name];
                    if (!!name && !foundedDep) {
                        const depPack = { name: name };
                        const index = dependenciesCounter++;
                        dependencies[index] = depPack;
                        dependencedPackages[name] = index;
                    }
                    if (!handledDepLinks[link.type_id]) {
                        const containValue = (_k = (_j = (_h = (_g = link === null || link === void 0 ? void 0 : link.type) === null || _g === void 0 ? void 0 : _g.contains) === null || _h === void 0 ? void 0 : _h[0]) === null || _j === void 0 ? void 0 : _j.value) === null || _k === void 0 ? void 0 : _k.value;
                        if (!containValue)
                            pckg.errors.push(`Link contain from package to ${(_l = link === null || link === void 0 ? void 0 : link.type) === null || _l === void 0 ? void 0 : _l.id} does not have value.`);
                        links.push({
                            id: counter++,
                            package: { dependencyId: dependencedPackages[name], containValue },
                        });
                        handledDepLinks[link.type_id] = true;
                    }
                }
                if (!!link.from_id && !globalLinks.byId[link.from_id]) {
                    const name = (_r = (_q = (_p = (_o = (_m = link === null || link === void 0 ? void 0 : link.from) === null || _m === void 0 ? void 0 : _m.contains) === null || _o === void 0 ? void 0 : _o[0]) === null || _p === void 0 ? void 0 : _p.package) === null || _q === void 0 ? void 0 : _q.value) === null || _r === void 0 ? void 0 : _r.value;
                    const foundedDep = dependencedPackages[name];
                    if (!!name && !foundedDep) {
                        const depPack = { name: name };
                        const index = dependenciesCounter++;
                        dependencies[index] = depPack;
                        dependencedPackages[name] = index;
                    }
                    if (!handledDepLinks[link.from_id]) {
                        const containValue = (_v = (_u = (_t = (_s = link === null || link === void 0 ? void 0 : link.from) === null || _s === void 0 ? void 0 : _s.contains) === null || _t === void 0 ? void 0 : _t[0]) === null || _u === void 0 ? void 0 : _u.value) === null || _v === void 0 ? void 0 : _v.value;
                        if (!containValue)
                            pckg.errors.push(`Link contain from package to ${(_w = link === null || link === void 0 ? void 0 : link.from) === null || _w === void 0 ? void 0 : _w.id} does not have value.`);
                        links.push({
                            id: counter++,
                            package: { dependencyId: dependencedPackages[name], containValue },
                        });
                        handledDepLinks[link.from_id] = true;
                    }
                }
                if (!!link.to_id && !globalLinks.byId[link.to_id]) {
                    const name = (_1 = (_0 = (_z = (_y = (_x = link === null || link === void 0 ? void 0 : link.to) === null || _x === void 0 ? void 0 : _x.contains) === null || _y === void 0 ? void 0 : _y[0]) === null || _z === void 0 ? void 0 : _z.package) === null || _0 === void 0 ? void 0 : _0.value) === null || _1 === void 0 ? void 0 : _1.value;
                    const foundedDep = dependencedPackages[name];
                    if (!!name && !foundedDep) {
                        const depPack = { name: name };
                        const index = dependenciesCounter++;
                        dependencies[index] = depPack;
                        dependencedPackages[name] = index;
                    }
                    if (!handledDepLinks[link.to_id]) {
                        const containValue = (_5 = (_4 = (_3 = (_2 = link === null || link === void 0 ? void 0 : link.to) === null || _2 === void 0 ? void 0 : _2.contains) === null || _3 === void 0 ? void 0 : _3[0]) === null || _4 === void 0 ? void 0 : _4.value) === null || _5 === void 0 ? void 0 : _5.value;
                        if (!containValue)
                            pckg.errors.push(`Link contain from package to ${(_6 = link === null || link === void 0 ? void 0 : link.to) === null || _6 === void 0 ? void 0 : _6.id} does not have value.`);
                        links.push({
                            id: counter++,
                            package: { dependencyId: dependencedPackages[name], containValue },
                        });
                        handledDepLinks[link.to_id] = true;
                    }
                }
            }
            pckg.dependencies = dependencies;
            for (let l = 0; l < globalLinks.links.length; l++) {
                const link = globalLinks.links[l];
                if (link.type_id === global_ids_1.GLOBAL_ID_CONTAIN) {
                    containsByTo[+link.to_id] = link;
                }
                else if (link.id !== options.packageLinkId) {
                    links.push(link);
                }
            }
            for (let l = 0; l < links.length; l++) {
                const link = links[l];
                link._id = link.id;
                if (!link.package) {
                    link.id = ((_8 = (_7 = containsByTo[+link.id]) === null || _7 === void 0 ? void 0 : _7.value) === null || _8 === void 0 ? void 0 : _8.value) ? containsByTo[+link.id].value.value : counter++;
                    for (let li = 0; li < ((_9 = link === null || link === void 0 ? void 0 : link.out) === null || _9 === void 0 ? void 0 : _9.length); li++) {
                        const _link = link.out[li];
                        _link.from_id = link.id;
                    }
                    for (let li = 0; li < ((_10 = link === null || link === void 0 ? void 0 : link.in) === null || _10 === void 0 ? void 0 : _10.length); li++) {
                        const _link = link.in[li];
                        _link.to_id = link.id;
                    }
                    for (let li = 0; li < ((_11 = link === null || link === void 0 ? void 0 : link.typed) === null || _11 === void 0 ? void 0 : _11.length); li++) {
                        const _link = link.typed[li];
                        _link.type_id = link.id;
                    }
                    let value;
                    if (link === null || link === void 0 ? void 0 : link.value) {
                        const _12 = link.value, { id: __id, link_id: __link_id } = _12, _value = __rest(_12, ["id", "link_id"]);
                        value = _value;
                    }
                    const newLink = {
                        id: link.id,
                        type: link.type_id,
                    };
                    if (link.from)
                        newLink.from = link.from.id;
                    if (link.to)
                        newLink.to = link.to.id;
                    if (link.package)
                        newLink.package = link.package;
                    if (value)
                        newLink.value = value;
                    pckg.data.push(newLink);
                }
                else {
                    const newLink = {
                        id: link.id,
                        package: link.package,
                    };
                    pckg.data.push(newLink);
                }
            }
        });
    }
    export(options) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const globalLinks = yield this.selectLinks(options);
            const packageLink = (_a = globalLinks.links) === null || _a === void 0 ? void 0 : _a.find(l => { var _a, _b; return ((_b = (_a = l === null || l === void 0 ? void 0 : l.type) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.value) === global_ids_1.GLOBAL_NAME_PACKAGE; });
            const packageName = (_b = packageLink === null || packageLink === void 0 ? void 0 : packageLink.value) === null || _b === void 0 ? void 0 : _b.value;
            const pckg = {
                package: { name: packageName },
                data: [],
                strict: false,
                errors: [],
            };
            yield this.serialize(globalLinks, options, pckg);
            return pckg;
        });
    }
}
exports.Packager = Packager;
//# sourceMappingURL=packager.js.map